@@ -584,7 +584,7 @@ void        *ssl_config_perdir_merge(pool *, void *, void *);
 const char  *ssl_cmd_SSLMutex(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLPassPhraseDialog(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLRandomSeed(cmd_parms *, char *, char *, char *, char *);
-const char  *ssl_cmd_SSLEngine(cmd_parms *, char *, int flag);
+const char  *ssl_cmd_SSLEngine(cmd_parms *, char *, int);
 const char  *ssl_cmd_SSLCipherSuite(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLCertificateFile(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLCertificateKeyFile(cmd_parms *, char *, char *);
@@ -593,7 +593,7 @@ const char  *ssl_cmd_SSLCACertificatePath(cmd_parms *, SSLDirConfigRec *, char *
 const char  *ssl_cmd_SSLCACertificateFile(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLCARevocationPath(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLCARevocationFile(cmd_parms *, SSLDirConfigRec *, char *);
-const char  *ssl_cmd_SSLVerifyClient(cmd_parms *, SSLDirConfigRec *, char *level);
+const char  *ssl_cmd_SSLVerifyClient(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLVerifyDepth(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLSessionCache(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLSessionCacheTimeout(cmd_parms *, char *, char *);
@@ -611,12 +611,13 @@ void         ssl_init_ConfigureServer(server_rec *, pool *, SSLSrvConfigRec *);
 STACK_OF(X509_NAME) 
             *ssl_init_FindCAList(server_rec *, pool *, char *, char *);
 void         ssl_init_Child(server_rec *, pool *);
-void         ssl_init_ModuleKill(void *data);
+void         ssl_init_ChildKill(void *);
+void         ssl_init_ModuleKill(void *);
 
 /*  Apache API hooks  */
 void         ssl_hook_AddModule(module *);
 void         ssl_hook_RemoveModule(module *);
-char        *ssl_hook_RewriteCommand(cmd_parms *, void *config, const char *);
+char        *ssl_hook_RewriteCommand(cmd_parms *, void *, const char *);
 void         ssl_hook_NewConnection(conn_rec *);
 void         ssl_hook_TimeoutConnection(int);
 void         ssl_hook_CloseConnection(conn_rec *);
@@ -686,9 +687,10 @@ void          ssl_ds_table_kill(ssl_ds_table *);
 
 /*  Mutex Support  */
 void         ssl_mutex_init(server_rec *, pool *);
-void         ssl_mutex_open(server_rec *, pool *);
+void         ssl_mutex_reinit(server_rec *, pool *);
 void         ssl_mutex_on(server_rec *);
 void         ssl_mutex_off(server_rec *);
+void         ssl_mutex_kill(server_rec *s);
 void         ssl_mutex_file_create(server_rec *, pool *);
 void         ssl_mutex_file_open(server_rec *, pool *);
 void         ssl_mutex_file_remove(void *);
@@ -702,7 +704,7 @@ BOOL         ssl_mutex_sem_release(void);
 
 /*  Logfile Support  */
 void         ssl_log_open(server_rec *, server_rec *, pool *);
-BOOL         ssl_log_applies(server_rec *s, int level);
+BOOL         ssl_log_applies(server_rec *, int);
 void         ssl_log(server_rec *, int, const char *, ...);
 void         ssl_die(void);
 
@@ -715,6 +717,9 @@ char        *ssl_var_lookup(pool *, server_rec *, conn_rec *, request_rec *, cha
 void         ssl_io_register(void);
 void         ssl_io_unregister(void);
 long         ssl_io_data_cb(BIO *, int, const char *, int, long, long);
+#ifdef SSL_EXPERIMENTAL
+void         ssl_io_suck(request_rec *, SSL *);
+#endif
 
 /*  PRNG  */
 int          ssl_rand_seed(server_rec *, pool *, ssl_rsctx_t);
