@@ -130,7 +130,7 @@ void ssl_init_Module(server_rec *s, pool *p)
     /*
      * Let us cleanup on restarts and exists
      */
-    ap_register_cleanup(p, s, ssl_init_ModuleKill, ap_null_cleanup);
+    ap_register_cleanup(p, s, ssl_init_ModuleKill, ssl_init_ChildKill);
 
     /*
      * Any init round fixes the global config
@@ -756,19 +756,26 @@ STACK_OF(X509_NAME) *ssl_init_FindCAList(server_rec *s, pool *pp, char *cpCAfile
 void ssl_init_Child(server_rec *s, pool *p)
 {
      /* open the mutex lockfile */
-     ssl_mutex_open(s, p);
+     ssl_mutex_reinit(s, p);
      return;
 }
 
+void ssl_init_ChildKill(void *data)
+{
+    /* currently nothing to do */
+    return;
+}
+
 void ssl_init_ModuleKill(void *data)
 {
     SSLSrvConfigRec *sc;
     server_rec *s = (server_rec *)data;
 
     /*
-     * Drop the session cache
+     * Drop the session cache and mutex
      */
     ssl_scache_kill(s);
+    ssl_mutex_kill(s);
 
     /*
      * Free the non-pool allocated structures
