@@ -105,7 +105,11 @@ X509 *SSL_read_X509(FILE *fp, X509 **x509, int (*cb)())
     BIO *bioF;
 
     /* 1. try PEM (= DER+Base64+headers) */
+#if SSL_LIBRARY_VERSION < 0x00904000
     rc = PEM_read_X509(fp, x509, cb);
+#else
+    rc = PEM_read_X509(fp, x509, cb, NULL);
+#endif
     if (rc == NULL) {
         /* 2. try DER+Base64 */
         fseek(fp, 0L, SEEK_SET);
@@ -152,7 +156,11 @@ EVP_PKEY *SSL_read_PrivateKey(FILE *fp, EVP_PKEY **key, int (*cb)())
     BIO *bioF;
 
     /* 1. try PEM (= DER+Base64+headers) */
+#if SSL_LIBRARY_VERSION < 0x00904000
     rc = PEM_read_PrivateKey(fp, key, cb);
+#else
+    rc = PEM_read_PrivateKey(fp, key, cb, NULL);
+#endif
     if (rc == NULL) {
         /* 2. try DER+Base64 */
         fseek(fp, 0L, SEEK_SET);
@@ -385,7 +393,11 @@ int SSL_CTX_use_certificate_chain(
     }
     /* optionally skip a leading server certificate */
     if (skipfirst) {
+#if SSL_LIBRARY_VERSION < 0x00904000
         if ((x509 = PEM_read_bio_X509(bio, NULL, cb)) == NULL) {
+#else
+        if ((x509 = PEM_read_bio_X509(bio, NULL, cb, NULL)) == NULL) {
+#endif
             BIO_free(bio);
             return -1;
         }
@@ -398,7 +410,11 @@ int SSL_CTX_use_certificate_chain(
     }
     /* create new extra chain by loading the certs */
     n = 0;
+#if SSL_LIBRARY_VERSION < 0x00904000
     while ((x509 = PEM_read_bio_X509(bio, NULL, cb)) != NULL) {
+#else
+    while ((x509 = PEM_read_bio_X509(bio, NULL, cb, NULL)) != NULL) {
+#endif
         if (!SSL_CTX_add_extra_chain_cert(ctx, x509)) { 
             X509_free(x509);
             BIO_free(bio);
